DOC_URL = ''


class ResponseError(Exception):
    doc_url = None

    type = None

    def __init__(self, message, type, docUrl=''):
        super().__init__(message)
        if len(docUrl) > 0:
            self.doc_url = docUrl
        else:
            self.doc_url = '{}/{}'.format(DOC_URL, type)
        self.type = type


class ParameterValidationError(ResponseError, Exception):
    def __init__(self, **error):
        super().__init__(type='parameter-validation-error', **error)



class GetCategoriesForUniverse:
    def get_categories_for_universe(self, universe_id: int):
        request_params = {
            'method': 'get',
            'path': f'/api/signals/universes/{universe_id}/categories'
        }

        response = self.request(**request_params)

        return response


class Events:
    def __init__(self):
        self.events = {}

    def on(self, event_name, handler):
        if event_name not in self.events:
            self.events[event_name] = []
        self.events[event_name].append(handler)
        handler_id = len(self.events[event_name]) - 1
        return handler_id

    def off(self, event_name, handler):
        if event_name in self.events and handler in self.events[event_name]:
            self.events[event_name].remove(handler)
            return True
        return False

    def emit(self, event_name, *args, **keywargs):
        if event_name in self.events:
            for handler in self.events[event_name]:
                handler(*args, **keywargs)



import re
import json
import requests
from clearmacro.errors import ResponseError
from clearmacro.configuration import Configuration
from clearmacro.events import Events
from clearmacro.operations import Operations
from clearmacro.version import VERSION


class Client(Events, Operations):
    configuration = None

    user_agent = 'ClearMacro/python@{}'.format(VERSION)

    def __init__(self, **configuration_overrides):
        super().__init__()
        self.configuration = Configuration(**configuration_overrides)
        self.on("request", lambda req: self.configuration.logger.debug('Request: %s', req))
        self.on("response", lambda res: self.configuration.logger.debug('Request: %s', res))
        self.on("error", lambda err: self.configuration.logger.error(err))
        self.configuration.credentials = self.login(self.configuration.username, self.configuration.password)

    def request(self, path, method, **params):
        url = self.build_request_url(path)
        request_params = {**params, **self.configuration.request_params()}
        self.emit('request', {'url': url, **request_params})
        response = self.call_request(url, method, request_params)
        data = response.json()
        self.emit('response', data)
        return data

    def call_request(self, url, method, params={}):
        try:
            request = getattr(requests, method)
            response = request(url, **params)
            if response.status_code == requests.codes.ok:
                return response
            try:
                raise ResponseError(**response.json())
            except json.decoder.JSONDecodeError:
                raise ResponseError(response, type='network-error')
        except requests.exceptions.Timeout:
            raise ResponseError('Connection timed out', type='timeout')
        except requests.exceptions.RequestException as e:
            raise ResponseError(str(e), type='liberror')

    def build_request_url(self, path):
        request_url = self.configuration.url + path
        safe_request_url = re.sub(r"(?<!:)\/\/", '/', request_url)
        return safe_request_url



from pathlib import PurePath
import logging


class Configuration:
    username = None

    password = None

    credentials = None

    url = "https://api.clearmacro.com"

    logger = logging.getLogger("clearmacro")

    # True, False or path to CA Bundle file
    ssl_verify = True

    # None or a single file (containing the private key and
    # the certificate) or as a tuple of both files’ paths
    cert_file = None

    debug = False

    timeout = 30

    def __init__(self, **config):
        for key, value in config.items():
            setattr(self, key, value)

    def request_params(self):
        auth = {}

        if (self.credentials is not None):
            auth = {
                'authorization': 'Bearer {}'.format(self.credentials.get('access_token'))
            }

        params = {
            'timeout': self.timeout,
            'verify': self.ssl_verify,
            'headers': {
                **auth,
                'content-type': 'application/json',
                'accept': 'application/json',
            }
        }

        if self.cert_file:
            params['cert'] = self.cert_file

        return params





class Login:
    def login(self, username, password):
        data = {
            'email': username,
            'password': password,
        }

        request_params = {
            'method': 'post',
            'path': '/api/auth/login',
            'json': data,
        }

        self.configuration.logger.debug('Executing operation login: %s', request_params)

        self.emit('operation:login', request_params)

        response = self.request(**request_params)

        return response



from clearmacro.operations.login import Login
from clearmacro.operations.get_categories_for_universe import GetCategoriesForUniverse
from clearmacro.operations.list_markets_for_signal import ListMarketsForSignal

class Operations(Login, ListMarketsForSignal, GetCategoriesForUniverse, object):
    pass



import os
from codecs import open
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

os.chdir(here)

version_contents = {}
with open(os.path.join(here, "clearmacro", "version.py"), encoding="utf-8") as f:
    exec(f.read(), version_contents)

setup(
    name='cmqd',
    version=version_contents["VERSION"],
    url='https://github.com/cmqd/cm-api-python-sdk',
    license='MIT',
    keywords="cmqd clearmacro python api sdk",
    author='ClearMacro',
    author_email='support@clearmacro.com',
    description="Python bindings for the ClearMacro API",
    packages=find_packages(exclude=['tests', 'tests.*']),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"',
        'requests[security] >= 2.20; python_version < "3.0"',
    ],
    project_urls={
        "Bug Tracker": "https://github.com/cmqd/cm-api-python-sdk/issues",
        "Documentation": "https://github.com/cmqd/cm-api-python-sdk",
        "Source Code": "https://github.com/cmqd/cm-api-python-sdk",
    },
    zip_safe=False
)



class ListMarketsForSignal:
    def list_markets_for_signal(self, signal_id: int):
        request_params = {
            'method': 'get',
            'path': f'/api/signals/{signal_id}/markets'
        }

        response = self.request(**request_params)

        return response